import pytest
from pytest_lazyfixture import lazy_fixture

from mati.types import ValidationInputType


def test_type_to_str():
    assert str(ValidationInputType.document_photo) == 'document-photo'


@pytest.mark.parametrize(
    ('verification_document', 'expected_type'),
    (
        (
            lazy_fixture('verification_document_national_id'),
            'ine',
        ),
        (
            lazy_fixture('verification_document_passport'),
            'passport',
        ),
        (
            lazy_fixture('verification_document_dni'),
            'dni',
        ),
        (
            lazy_fixture('verification_document_foreign_id'),
            'foreign-id',
        ),
    ),
)
def test_document_type(verification_document, expected_type):
    assert verification_document.document_type == expected_type



from dataclasses import dataclass, field
from enum import Enum
from typing import BinaryIO, Dict, List, Optional, Union


class SerializableEnum(str, Enum):
    def __str__(self) -> str:
        return self.value


class PageType(SerializableEnum):
    front = 'front'
    back = 'back'


class ValidationInputType(SerializableEnum):
    document_photo = 'document-photo'
    selfie_photo = 'selfie-photo'
    selfie_video = 'selfie-video'


class ValidationType(SerializableEnum):
    driving_license = 'driving-license'
    national_id = 'national-id'
    passport = 'passport'
    proof_of_residency = 'proof-of-residency'


@dataclass
class VerificationDocumentStep:
    id: str
    status: int
    error: Optional[Dict] = None
    data: Optional[Dict] = field(default_factory=dict)


@dataclass
class VerificationDocument:
    country: str
    region: str
    photos: List[str]
    steps: List[VerificationDocumentStep]
    type: str
    fields: Optional[dict] = None

    @property
    def document_type(self) -> str:
        if self.type in ['national-id', 'passport']:
            document_data = [
                step.data
                for step in self.steps
                if step.id == 'document-reading'
            ]
            if document_data[-1] is not None:
                if (
                    all(
                        [
                            self.type == 'national-id',
                            document_data,
                            'cde' in document_data[-1],
                        ]
                    )
                    and document_data[-1]['cde']['label'] == 'Elector Key'
                    and document_data[-1]['cde']['value']
                ):
                    return 'ine'
                elif self.type == 'passport':
                    return 'passport'
                else:
                    return 'dni'
        return self.type

    @property
    def address(self) -> str:
        """
        This property fills the address direct from the ocr fields `address`
        """
        if self.fields and 'address' in self.fields:
            return self.fields['address']['value']
        return ''

    @property
    def full_name(self) -> str:
        """
        This property fills the fullname direct from the ocr fields `full_name`
        """
        if self.fields and 'full_name' in self.fields:
            return self.fields['full_name']['value']
        return ''

    @property
    def curp(self) -> str:
        """
        This property fills the CURP direct from the ocr fields `curp`
        """
        if self.fields and 'curp' in self.fields:
            return self.fields['curp']['value']
        return ''


@dataclass
class LivenessMedia:
    video_url: str
    sprite_url: str
    selfie_url: str


@dataclass
class Liveness:
    status: int
    id: str
    data: LivenessMedia
    error: Optional[Dict]


@dataclass
class DocumentScore:
    is_valid: bool = False
    score: int = 0
    error_codes: Optional[List[str]] = None


@dataclass
class UserValidationFile:
    filename: str
    content: BinaryIO
    input_type: Union[str, ValidationInputType]
    validation_type: Union[str, ValidationType] = ''
    country: str = ''  # alpha-2 code: https://www.iban.com/country-codes
    region: str = ''  # 2-digit US State code (if applicable)
    group: int = 0
    page: Union[str, PageType] = PageType.front



import pytest
from requests.exceptions import HTTPError

from mati import Client


@pytest.mark.vcr
def test_client_renew_access_token():
    client = Client('api_key', 'secret_key')
    assert client.bearer_token is None
    client.get_valid_bearer_token()
    assert not client.bearer_token.expired
    assert client.bearer_token == client.get_valid_bearer_token()


@pytest.mark.vcr
def test_failed_auth():
    client = Client('wrong', 'creds')
    with pytest.raises(HTTPError) as exc_info:
        client.access_tokens.create()
    assert exc_info.value.response.status_code == 401



from dataclasses import dataclass, fields
from typing import ClassVar, Dict, Union

import iso8601

from mati.helpers import camel_to_underscore, change_dict_naming_convention


@dataclass
class Resource:
    _client: ClassVar['mati.Client']  # type: ignore
    _endpoint: ClassVar[str]

    # purely for MyPy
    def __init__(self, **_) -> None:  # pragma: no cover
        ...

    def __post_init__(self) -> None:
        for attr, value in self.__dict__.items():
            if attr.startswith('date'):
                setattr(self, attr, iso8601.parse_date(value))

    @classmethod
    def _from_dict(cls, obj_dict: Dict[str, Union[str, int]]) -> 'Resource':
        new_dict = change_dict_naming_convention(obj_dict, camel_to_underscore)
        cls._filter_excess_fields(new_dict)
        return cls(**new_dict)

    @classmethod
    def _filter_excess_fields(cls, obj_dict: Dict) -> None:
        """
        dataclasses don't allow __init__ to be called with excess fields. This
        method allows the API to add fields in the response body without
        breaking the client
        """
        excess = set(obj_dict.keys()) - {f.name for f in fields(cls)}
        for f in excess:
            del obj_dict[f]



import pytest

from mati import Client
from mati.resources import Resource


@pytest.mark.vcr()
def test_two_clients():
    secondary_client = Client('test', 'test')
    main_client = Client('api_key', 'secret_key')

    # NOTE: Resource._client is the default client for requests and
    # always has the reference to the last client initialized
    assert Resource._client == main_client

    assert main_client.bearer_token is None
    assert secondary_client.bearer_token is None

    metadata = dict(
        nombres='Georg Wilhelm',
        primer_apellido='Friedrich',
        segundo_apellido='Hegel',
        dob='1770-08-27',
    )

    main_identity = main_client.identities.create(**metadata)
    secondary_identity = secondary_client.identities.create(
        client=secondary_client, **metadata
    )

    main_retrieve = main_client.identities.retrieve(main_identity.id)
    secondary_retrieve = secondary_client.identities.retrieve(
        secondary_identity.id, client=secondary_client
    )
    assert main_retrieve.id == main_identity.id
    assert secondary_retrieve.id == secondary_identity.id

    assert main_client.bearer_token == main_client.get_valid_bearer_token()
